using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Text;
using System.Xml;
using System.Linq;
using System.Collections.Generic;
using Ato.EN.IntegrationServices.CodeGenerationIITR;

namespace Ato.EN.IntegrationServices.CodeGenerationDDCTNS
{
    public class ManualDDCTNS2019Writer : IManualDDCTNS2019Writer
    {
        // ------------------------------------------------------------------------------
        // The methods in this interface must be implemented in a manually written class,
        // as ESR does not currently support some aspect of their mapping

        // An implementation of this class must be provided to the main mapping method.

        // Each implemented method will likely want to make use of the DDCTNS2019Writer.AddFormLineItem method
        // however, if you do not use that method, you must remember to increment lineItemCount after adding a line.
        // ------------------------------------------------------------------------------
        private IITR2019 iitr2019;

        public ManualDDCTNS2019Writer(IITR2019 parentReport)
        {
            iitr2019 = parentReport;
        }


        // MAP11525IITR136

        // Element: IITR136
        // Set Target to ^IITR136

        // Update Comment
        // PBI 1126325 : Wildcards : Sprint 16.6

        public void MAP11525IITR136(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR136.HasValue)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR136.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP11545DDCTNS415

        // Element: DDCTNS415
        // Set Target to ENCODE(
        // FirstOccurenceOf(^DDCTNS415, Max( InSet(
        //       Sum(^DDCTNS416 where ^DDCTNS415 = "Compulsory work uniform"), 
        //       Sum(^DDCTNS416 where ^DDCTNS415 = "Non-compulsory work uniform"), 
        //       Sum(^DDCTNS416 where ^DDCTNS415 = "Protective clothing"), 
        //       Sum(^DDCTNS416 where ^DDCTNS415 = "Occupation specific clothing") 
        // ))), GRP-WORK-RELATED-CLOTH-EXP-TYPE, "Other Codes")

        // If each repeating instance of 'Work-related clothing expenses' uses the same claim type, set to the corresponding claim type code.
        // 
        // Where more than one method has been used, assign the code for the method that has generated the largest amount measured in dollars. If the amount is the same set to the first code entered.

        // Update Comment
        // PBI1126326 : Wildcards : Sprint 17.1; New Mapping Rule

        public void MAP11545DDCTNS415(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            Dictionary<string, decimal> totals = new Dictionary<string, decimal>();

            if (report.RPClthngSeqNumContextCollection != null && report.RPClthngSeqNumContextCollection.Count() > 0)
            {

                // Loop through the contexts and add up the totals, only adding the type if it has been encountered (and in the order they are encountered)
                foreach (DDCTNS2019.RPClthngSeqNumContext rpClthngSeqNumContext in report.RPClthngSeqNumContextCollection)
                {
                    if (rpClthngSeqNumContext.DDCTNS415 != null && rpClthngSeqNumContext.DDCTNS416.HasValue)
                    {
                        string currentCode = rpClthngSeqNumContext.DDCTNS415.ToUpperInvariant();
                        if (totals.ContainsKey(currentCode))
                            totals[currentCode] += rpClthngSeqNumContext.DDCTNS416.Value;
                        else
                            totals.Add(currentCode, rpClthngSeqNumContext.DDCTNS416.Value);
                    }
                }

                // As long as one of the codes was returned, find the highest value one and map it using the code table
                if (totals.Count != 0)
                {
                    KeyValuePair<string, decimal> highestValueCode = totals.First();
                    
                    foreach (KeyValuePair<string, decimal> currentCode in totals)
                    {
                        // because we want to map the first one if multiple are highest, only swap if the current is greater than the highest
                        if (currentCode.Value > highestValueCode.Value)
                        {
                            highestValueCode = currentCode;
                        }
                    }

                    string mappedValue = null;
                    #region Mapping of GRP-WORK-RELATED-CLOTH-EXP-TYPE
                    switch (highestValueCode.Key)
                    {
                        case @"COMPULSORY WORK UNIFORM": mappedValue = @"25"; break;
                        case @"NON-COMPULSORY WORK UNIFORM": mappedValue = @"15"; break;
                        case @"PROTECTIVE CLOTHING": mappedValue = @"10"; break;
                        case @"OCCUPATION SPECIFIC CLOTHING": mappedValue = @"20"; break;
                    }
                    #endregion // Mapping of GRP-WORK-RELATED-CLOTH-EXP-TYPE
                    if (mappedValue != null && mappedValue.Length > 0)
                        DDCTNS2019Writer.AddFormLineItem(writer, mappedValue, sectionName, fieldName, fieldUsage, length, repeatCount, action, ref lineItemCount);

                }
                
            }

        }

        // MAP11526IITR137

        // Element: IITR137
        // Set Target to ^IITR137

        // Update Comment
        // PBI1126326 : Wildcards : Sprint 17.1; New Mapping Rule

        public void MAP11526IITR137(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR137.HasValue)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR137.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }


        // MAP25515DDCTNS121

        // Element: DDCTNS121
        // IF ^ DDCTNS121 = "Cents per kilometre"
        //      Set Target to ^ DDCTNS122 * 0.68
        // ELSE
        //      Set Target to RoundDown((^ DDCTNS124 + ^ DDCTNS125) * (RoundDown(^ DDCTNS123, 2) / 100), 2)
        // ENDIF

        // Update Comment
        // 10/07/2019 TFSD1482157 Code Blue; Sprint 19.4
        // Updated to include RoundDown on decimal and percent fields
        // From: IF ^ DDCTNS121 = "Cents per kilometre"
        //      Set Target to ^ DDCTNS122 * 0.68
        // ELSE
        //      Set Target to(^ DDCTNS124 + ^ DDCTNS125) *(^ DDCTNS123 / 100)
        // ENDIF

        public void MAP25515DDCTNS121(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (report.RPEduSeqNumContextCollection != null && report.RPEduSeqNumContextCollection.Count() > 0)
            {
                DDCTNS2019.RPEduSeqNumContext rpEduSeqNumContext = report.RPEduSeqNumContextCollection[repeatCount - 1];

                if (rpEduSeqNumContext.DDCTNS121 != null)
                {
                    decimal ratePerKilometre = 0.68M;
                    decimal carExpenseBusinessUsePercentage = 0;

                    if (rpEduSeqNumContext.DDCTNS121.ToUpperInvariant() == "CENTS PER KILOMETRE")
                        // Cents per Kilometre = Kilometres Travelled (DDCTNS122) times the rate per kilometre (0.68), truncated to 2 decimal places (i.e. 12.689 becomes 12.68)
                        DDCTNS2019Writer.AddFormLineItem(writer, Math.Truncate(100 * (rpEduSeqNumContext.DDCTNS122.GetValueOrDefault() * ratePerKilometre)) / 100, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
                    else if (rpEduSeqNumContext.DDCTNS121.ToUpperInvariant() == "LOGBOOK")
                    {
                        // Calulating the percentage of the rule - (RoundDown(^DDCTNS123, 2) / 100)
                        carExpenseBusinessUsePercentage = (Math.Truncate(100 * rpEduSeqNumContext.DDCTNS123.GetValueOrDefault()) / 100) / 100; 

                        // Logbook Amount = Expenses Amount (DDCTNS124) + Decline in Value Amount (DDCTNS125) times the business use percentage (DDCTNS123 / 100), truncated to 2 decimal places (i.e. 12.689 becomes 12.68)
                        DDCTNS2019Writer.AddFormLineItem(writer, Math.Truncate(100 * ((rpEduSeqNumContext.DDCTNS124.GetValueOrDefault() + rpEduSeqNumContext.DDCTNS125.GetValueOrDefault()) * carExpenseBusinessUsePercentage)) / 100, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
                    }
                }
            }
        }

        // MAP27279IITR148

        // Element: IITR148
        // Set Target to ^IITR148

        // Update Comment
        // PBI1126357 : Wildcards : Sprint 17.1; New Mapping Rule

        public void MAP27279IITR148(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR148 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR148.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP29149IITR502

        // Element: IITR502
        // Set Target to ^IITR502

        // Update Comment
        // PBI 1126929, One Team 16.5

        public void MAP29149IITR502(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR502 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR502.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP29150IITR503

        // Element: IITR503
        // Set Target to ^IITR503

        // Update Comment
        // PBI 1126929, One Team 16.5

        public void MAP29150IITR503(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR503 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR503.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP29151IITR504

        // Element: IITR504
        // Set Target to ^IITR504

        // Update Comment
        // PBI 1126929, One Team 16.5

        public void MAP29151IITR504(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR504 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR504.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP27278DDCTNS204

        // Element: DDCTNS204
        // Set Target to Sum(ConditionalValue(InSet(^DDCTNS202,'"Other", "Decline in value"'), ^DDCTNS204, 0))

        // Update Comment
        // 15/11/2018 PBI1126327 Alpha Team Sprint 16.6; NEW

        public void MAP27278DDCTNS204(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            decimal result = 0;

            if (report.RPEduSeqNumContextCollection != null && report.RPEduSeqNumContextCollection.Count() > 0)
            {
                foreach (DDCTNS2019.RPEduSeqNumContext temp in report.RPEduSeqNumContextCollection)
                {
                    if (temp.DDCTNS202 != null && temp.DDCTNS204.HasValue)
                    {
                        if (temp.DDCTNS202.Equals("Other") || temp.DDCTNS202.Equals("Decline in value"))
                        {
                            result += temp.DDCTNS204.Value;
                        }
                    }
                }
                DDCTNS2019Writer.AddFormLineItem(writer, result, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
            }
        }

        // MAP11527IITR139

        // Element: IITR139
        // Set Target to ^IITR139

        // Update Comment
        // 9/11/2018 PBI1126327 Alpha Team Sprint 16.6; NEW

        public void MAP11527IITR139(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR139 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR139.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP20009IITR366

        // Element: IITR366
        // Set Target to ^IITR366

        // Update Comment
        // 16/11/2018 PBI1126369 Alpha Team Sprint 16.6; NEW

        public void MAP20009IITR366(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR366 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR366.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }


        // MAP11540IITR134

        // Element: IITR134
        // Set Target to ^IITR134

        // Update Comment
        // 15/11/2018 TFS1125086/TFS1126324 A Team Sprint 16.6; NEW

        public void MAP11540IITR134(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR134 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR134.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP11544DDCTNS105

        // Element: DDCTNS105
        // IF NotSameValues(^ DDCTNS102)
        //      IF RoundDown((Sum(^ DDCTNS105 + ^ DDCTNS106) *(RoundDown(^ DDCTNS104, 2) / 100)), 2) = (Sum(^ DDCTNS103 * 0.68))
        //          Set Target to ENCODE_TO_FDFGROUP(^ DDCTNS102[1], GRP - WORK - RELATED - CAR - EXP - TYPE, "Other Codes")
        //      ELSE IF RoundDown((Sum(^ DDCTNS105 + ^ DDCTNS106) * (RoundDown(^ DDCTNS104, 2) / 100)), 2) > (Sum(^ DDCTNS103 * 0.68))
        //          Set Target to "20"
        //      ELSE
        //          Set Target to "5"
        //      ENDIF
        // ELSE
        //      Set Target to ENCODE_TO_FDFGROUP(^ DDCTNS102[1], GRP - WORK - RELATED - CAR - EXP - TYPE, "Other Codes")
        // ENDIF

        // If each repeating instance of 'Work-related car expenses' uses the same calculation type, set to the corresponding calculation type code.
        // 
        // Where more than one method has been used, assign the code for the method that has generated the largest proportion, measured in dollars. If the amount is the same set to the first code entered.

        // Update Comment
        // 10/07/2019 TFSD1482157 Code Blue; Sprint 19.4
        // Updated to include RoundDown on decimal and percent fields
        // From: IF NotSameValues(^ DDCTNS102)
        //      IF(Sum(^ DDCTNS105 + ^ DDCTNS106) * (^ DDCTNS104 / 100)) = (Sum(^ DDCTNS103 * 0.68))
        //          Set Target to ENCODE_TO_FDFGROUP(^ DDCTNS102[1], GRP - WORK - RELATED - CAR - EXP - TYPE, "Other Codes")
        //      ELSE IF(Sum(^ DDCTNS105 + ^ DDCTNS106) *(^ DDCTNS104 / 100)) > (Sum(^ DDCTNS103 * 0.68))
        //          Set Target to "20"
        //      ELSE
        //          Set Target to "5"
        //      ENDIF
        // ELSE
        //      Set Target to ENCODE_TO_FDFGROUP(^ DDCTNS102[1], GRP - WORK - RELATED - CAR - EXP - TYPE, "Other Codes")
        // ENDIF

        public void MAP11544DDCTNS105(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            // If Car Expenses Context(s) exists and if DDCTNS102 contains a valid code ('Cents per kilometre' or 'Logbook' - function returns null if not)
            if (report.RPCarSeqNumContextCollection != null && GetFDFCode_GrpWorkRelatedCarExpenseType(report.RPCarSeqNumContextCollection.First().DDCTNS102) != null)
            {
                // If all instances of DDCTNS102 are not equal to eachother
                string firstDDCTNS102 = report.RPCarSeqNumContextCollection.First().DDCTNS102.ToString();
                if (!report.RPCarSeqNumContextCollection.All(carexpenses => carexpenses.DDCTNS102.ToString() == firstDDCTNS102))
                {
                    // Perform respective calculations by looping thought each car expenses context
                    decimal sumLogbook = 0;
                    decimal sumCentsPerKilometre = 0;
                    decimal ratePerKilometre = 0.68M;
                    decimal carExpenseBusinessUsePercentage = 0;
                    foreach (DDCTNS2019.RPCarSeqNumContext carexpenses in report.RPCarSeqNumContextCollection)
                    {
                        // Calulating the percentage of the rule - (RoundDown(^DDCTNS104, 2) / 100)
                        carExpenseBusinessUsePercentage = (Math.Truncate(100 * carexpenses.DDCTNS104.GetValueOrDefault()) / 100) / 100;
                        // Logbook Amount = Expenses Amount (DDCTNS105) + Decline in Value Amount (DDCTNS106) times the business use percentage (DDCTNS104 / 100)
                        sumLogbook += Math.Truncate(100 * ((carexpenses.DDCTNS105.GetValueOrDefault() + carexpenses.DDCTNS106.GetValueOrDefault()) * carExpenseBusinessUsePercentage)) / 100;
                        // Cents per Kilometre = Kilometres Travelled (DDCTNS103) times the rate per kilometre (0.68)
                        sumCentsPerKilometre += Math.Truncate(100 * (carexpenses.DDCTNS103.GetValueOrDefault() * ratePerKilometre)) / 100;
                    }

                    if (sumCentsPerKilometre == sumLogbook)
                        DDCTNS2019Writer.AddFormLineItem(writer, GetFDFCode_GrpWorkRelatedCarExpenseType(firstDDCTNS102), sectionName, fieldName, fieldUsage, length, repeatCount, action, ref lineItemCount);
                    else if (sumLogbook > sumCentsPerKilometre)
                        DDCTNS2019Writer.AddFormLineItem(writer, GetFDFCode_GrpWorkRelatedCarExpenseType("Logbook"), sectionName, fieldName, fieldUsage, length, repeatCount, action, ref lineItemCount);
                    else
                        DDCTNS2019Writer.AddFormLineItem(writer, GetFDFCode_GrpWorkRelatedCarExpenseType("Cents per kilometre"), sectionName, fieldName, fieldUsage, length, repeatCount, action, ref lineItemCount);
                }
                else
                {
                    DDCTNS2019Writer.AddFormLineItem(writer, GetFDFCode_GrpWorkRelatedCarExpenseType(firstDDCTNS102), sectionName, fieldName, fieldUsage, length, repeatCount, action, ref lineItemCount);
                }
            }
        }
        #region Mapping of GRP-WORK-RELATED-CAR-EXP-TYPE
        private string GetFDFCode_GrpWorkRelatedCarExpenseType(string fromValue)
        {
            if (fromValue != null)
            {
                fromValue = fromValue.ToUpperInvariant();
                string mappedValue = null;
                switch (fromValue)
                {
                    case @"CENTS PER KILOMETRE": mappedValue = @"5"; break;
                    case @"LOGBOOK": mappedValue = @"20"; break;
                }
                return mappedValue;
            }
            else
            {
                return null;
            }
        }
        #endregion

        // MAP25504DDCTNS102

        // Element: DDCTNS102
        // IF ^DDCTNS102="Cents per kilometre"
        //      Set Target to ^DDCTNS103* 0.68
        // ELSE
        //      Set Target to RoundDown((^DDCTNS105 + ^DDCTNS106) * (RoundDown(^DDCTNS104, 2) / 100), 2)
        // ENDIF

        // If Calculation method used is 'Cents per kilometre', set target by multiplying the number of kilometres by 0.68. Else if the method used is 'Logbook', set target by multiplying the sum of total expenses plus decline in value by % of business use.

        // Update Comment
        // 10/07/2019 TFSD1482157 Code Blue; Sprint 19.4
        // Updated to include RoundDown on decimal and percent fields
        // From:  IF ^DDCTNS102="Cents per kilometre"
        //      Set Target to ^DDCTNS103* 0.68
        // ELSE
        //      Set Target to(^DDCTNS105 + ^DDCTNS106) * (^DDCTNS104 / 100)
        // ENDIF

        public void MAP25504DDCTNS102(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (report.RPCarSeqNumContextCollection != null && report.RPCarSeqNumContextCollection.Count() > 0)
            {
                DDCTNS2019.RPCarSeqNumContext rpCarSeqNumContext = report.RPCarSeqNumContextCollection[repeatCount - 1];

                if (rpCarSeqNumContext.DDCTNS102 != null)
                {
                    decimal ratePerKilometre = 0.68M;
                    decimal carExpenseBusinessUsePercentage = 0;

                    if (rpCarSeqNumContext.DDCTNS102.ToUpperInvariant() == "CENTS PER KILOMETRE")
                        // Cents per Kilometre = Kilometres Travelled (DDCTNS103) times the rate per kilometre (0.68), truncated to 2 decimal places (i.e. 12.689 becomes 12.68)
                        DDCTNS2019Writer.AddFormLineItem(writer, Math.Truncate(100 * (rpCarSeqNumContext.DDCTNS103.GetValueOrDefault() * ratePerKilometre)) / 100, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
                    else if (rpCarSeqNumContext.DDCTNS102.ToUpperInvariant() == "LOGBOOK")
                    {
                        // Calulating the percentage of the rule - (RoundDown(^DDCTNS104, 2) / 100)
                        carExpenseBusinessUsePercentage = (Math.Truncate(100 * rpCarSeqNumContext.DDCTNS104.GetValueOrDefault()) / 100) / 100;

                        // Logbook Amount = Expenses Amount (DDCTNS105) + Decline in Value Amount (DDCTNS106) times the business use percentage (DDCTNS104 / 100), truncated to 2 decimal places (i.e. 12.689 becomes 12.68)
                        DDCTNS2019Writer.AddFormLineItem(writer, Math.Truncate(100 * ((rpCarSeqNumContext.DDCTNS105.GetValueOrDefault() + rpCarSeqNumContext.DDCTNS106.GetValueOrDefault()) * carExpenseBusinessUsePercentage)) / 100, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
                    }
                }
            }
        }

        // MAP27282IITR368

        // Element: IITR368
        // Set Target to ^IITR368

        // Update Comment
        // PBI1126371 : One Team : Sprint 16.6 : New Mapping Rule

        public void MAP27282IITR368(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR368 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR368.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP12342IITR145

        // Element: IITR145
        // Set Target to ^IITR145

        // Update Comment
        // 20/11/2018 PBI1127221 One Team 16.6; Replicated from IITR to DDCTNS

        public void MAP12342IITR145(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR145 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR145.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP27330IITR141

        // Element: IITR141
        // Set Target to ^IITR141

        // Update Comment
        // 26/11/2018 PBI1126328 One Team Sprint 17.1; NEW

        public void MAP27330IITR141(XmlWriter writer, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR141 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR141.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP70104IITR147

        // Element: IITR147
        // Set Target to ^IITR147

        // Update Comment
        // 26/11/2018 TheATeam PBI1126352 Sprint 17.1; NEW

        public void MAP70104IITR147(XmlWriter writer, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR147 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR147.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP15636IITR146

        // Element: IITR146
        // Set Target to ^IITR146

        // Update Comment
        // 26/11/2018 PBI1126328 One Team Sprint 17.1; NEW

        public void MAP15636IITR146(XmlWriter writer, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR146 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR146.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP11901IITR358

        // Element: IITR358
        // Set Target to ^IITR358

        // Update Comment
        // 30/11/2018 PBI1162486 One Team Sprint 17.1; NEW

        public void MAP11901IITR358(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR358 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR358.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP16792IITR153

        // Element: IITR153
        // Set Target to ^IITR153

        // Update Comment
        // 30/11/2018 PBI1162521 One Team Sprint 17.1; NEW

        public void MAP16792IITR153(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR153 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR153.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP16842IITR154

        // Element: IITR154
        // Set Target to ^IITR154

        // Update Comment
        // 30/11/2018 PBI1162521 One Team Sprint 17.1; NEW

        public void MAP16842IITR154(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR154 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR154.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP16844IITR155

        // Element: IITR155
        // Set Target to ^IITR155

        // Update Comment
        // 30/11/2018 PBI1162521 One Team Sprint 17.1; NEW

        public void MAP16844IITR155(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR155 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR155.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // MAP16854IITR156

        // Element: IITR156
        // Set Target to ^IITR156

        // Update Comment
        // 30/11/2018 PBI1162521 One Team Sprint 17.1; NEW

        public void MAP16854IITR156(XmlWriter writer, DDCTNS2019 report, List<object> parentChain, object value, string sectionName, string fieldName, string fieldUsage, int length, int repeatCount, string action, ref int lineItemCount)
        {
            if (iitr2019.IITR156 != null)
                DDCTNS2019Writer.AddFormLineItem(writer, iitr2019.IITR156.Value, sectionName, fieldName, fieldUsage, repeatCount, action, ref lineItemCount);
        }

        // Catch all function to perform any other fixes necessary
        public void OtherMappingsHook(XmlWriter writer, DDCTNS2019 report, ref int lineItemCount)
        {

        }
    }
}
